
#include <windows.h>
#include <commdlg.h>
#include <streams.h>
#include <initguid.h>
#include "bndpuids.h"
#include "bindump.h"



const AMOVIESETUP_MEDIATYPE sudPinTypes =
{
    &MEDIATYPE_NULL,       // Major type
    &MEDIASUBTYPE_NULL      // Minor type
};

const AMOVIESETUP_PIN sudpPins =
{ L"Input",             // Pins string name
  FALSE,                // Is it rendered
  FALSE,                // Is it an output
  FALSE,                // Are we allowed none
  FALSE,                // And allowed many
  &CLSID_NULL,          // Connects to filter
  L"Output",            // Connects to pin
  1,                    // Number of types
  &sudPinTypes          // Pin information
};

const AMOVIESETUP_FILTER sudBinDump =
{
    &CLSID_BinDump,             // Filter CLSID
    L"BinDump filter",          // String name
    MERIT_DO_NOT_USE,           // Filter merit
    1,                          // Number pins
    &sudpPins                // Pin information
};

// List of class IDs and creator functions for the class factory. This
// provides the link between the OLE entry point in the DLL and an object
// being created. The class factory will call the static CreateInstance

CFactoryTemplate g_Templates[] = {
    { L"BinDump filter"
    , &CLSID_BinDump
    , CBinDump::CreateInstance
    , NULL
    , &sudBinDump }
};
int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);


//
// DllRegisterServer
//
// Handles sample registry and unregistry
//
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );

} // DllRegisterServer


//
// DllUnregisterServer
//
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );

} // DllUnregisterServer


// Constructor

CBinDumpFilter::CBinDumpFilter(CBinDump *pDump,
                         LPUNKNOWN pUnk,
                         CCritSec *pLock,
                         HRESULT *phr) :
    CBaseFilter(NAME("CBinDumpFilter"), pUnk, pLock, CLSID_BinDump, phr),
    m_pDump(pDump)
{
}


//
// GetPin
//
CBasePin * CBinDumpFilter::GetPin(int n)
{
    if (n == 0) {
        return m_pDump->m_pPin;
    } else {
        return NULL;
    }
}


//
// GetPinCount
//
int CBinDumpFilter::GetPinCount()
{
    return 1;
}


//
// Stop
//
// Overriden to close the dump file
//
STDMETHODIMP CBinDumpFilter::Stop()
{
    CAutoLock cObjectLock(m_pLock);
    m_pDump->CloseFile();
    return CBaseFilter::Stop();
}


//
// Pause
//
// Overriden to open the dump file
//
STDMETHODIMP CBinDumpFilter::Pause()
{
    CAutoLock cObjectLock(m_pLock);
    m_pDump->OpenFile();
    return CBaseFilter::Pause();
}


//
// Run
//
// Overriden to open the dump file
//
STDMETHODIMP CBinDumpFilter::Run(REFERENCE_TIME tStart)
{
    CAutoLock cObjectLock(m_pLock);
    m_pDump->OpenFile();
    return CBaseFilter::Run(tStart);
}


//
//  Definition of CBinDumpInputPin
//
CBinDumpInputPin::CBinDumpInputPin(CBinDump *pDump,
                             LPUNKNOWN pUnk,
                             CBaseFilter *pFilter,
                             CCritSec *pLock,
                             CCritSec *pReceiveLock,
                             HRESULT *phr) :

    CRenderedInputPin(NAME("CBinDumpInputPin"),
                  pFilter,                   // Filter
                  pLock,                     // Locking
                  phr,                       // Return code
                  L"Input"),                 // Pin name
    m_pReceiveLock(pReceiveLock),
    m_pDump(pDump),
    m_tLast(0)
{
}


//
// CheckMediaType
//
// Check if the pin can support this specific proposed type and format
//
HRESULT CBinDumpInputPin::CheckMediaType(const CMediaType *pmt)
{
	m_pDump->WriteMediaType(pmt);

    return S_OK;
}

//
// BreakConnect
//
// Break a connection
//
HRESULT CBinDumpInputPin::BreakConnect()
{
    if (m_pDump->m_pPosition != NULL) {
        m_pDump->m_pPosition->ForceRefresh();
    }
    return CBaseInputPin::BreakConnect();
}


//
// Receive
//
// Do something with this media sample
//
STDMETHODIMP CBinDumpInputPin::Receive(IMediaSample *pSample)
{
    CAutoLock lck(m_pReceiveLock);
    PBYTE pbData;

    // Has the filter been stopped yet
    if (m_pDump->m_hFile == INVALID_HANDLE_VALUE) {
        return NOERROR;
    }

    // Copy the data to the file

    HRESULT hr = pSample->GetPointer(&pbData);
    if (FAILED(hr)) {
        return hr;
    }

	m_pDump->WriteBinary(pbData, pSample->GetActualDataLength());

	return NOERROR;
}


//
// EndOfStream
//
STDMETHODIMP CBinDumpInputPin::EndOfStream(void)
{
    CAutoLock lck(m_pReceiveLock);
    return CRenderedInputPin::EndOfStream();

} // EndOfStream


//
// NewSegment
//
// Called when we are seeked
//
STDMETHODIMP CBinDumpInputPin::NewSegment(REFERENCE_TIME tStart,
                                       REFERENCE_TIME tStop,
                                       double dRate)
{
    m_tLast = 0;
    return S_OK;

} // NewSegment


//
//  CBinDump class
//
CBinDump::CBinDump(LPUNKNOWN pUnk, HRESULT *phr) :
    CUnknown(NAME("CBinDump"), pUnk, phr),
    m_pFilter(NULL),
    m_pPin(NULL),
    m_pPosition(NULL),
    m_hFile(INVALID_HANDLE_VALUE),
    m_pFileName(0)
{
    m_pFilter = new CBinDumpFilter(this, GetOwner(), &m_Lock, phr);
    if (m_pFilter == NULL) {
        *phr = E_OUTOFMEMORY;
        return;
    }

    m_pPin = new CBinDumpInputPin(this,GetOwner(),
                               m_pFilter,
                               &m_Lock,
                               &m_ReceiveLock,
                               phr);
    if (m_pPin == NULL) {
        *phr = E_OUTOFMEMORY;
        return;
    }
}


//
// SetFileName
//
// Implemented for IFileSourceFilter support
//
STDMETHODIMP CBinDump::SetFileName(LPCOLESTR pszFileName,const AM_MEDIA_TYPE *pmt)
{
    // Is this a valid filename supplied

    CheckPointer(pszFileName,E_POINTER);
    if(wcslen(pszFileName) > MAX_PATH)
        return ERROR_FILENAME_EXCED_RANGE;

    // Take a copy of the filename

    m_pFileName = new WCHAR[1+lstrlenW(pszFileName)];
    if (m_pFileName == 0)
        return E_OUTOFMEMORY;
    lstrcpyW(m_pFileName,pszFileName);

    // Create the file then close it

    HRESULT hr = OpenFile();
    CloseFile();
    return hr;

} // SetFileName


//
// GetCurFile
//
// Implemented for IFileSourceFilter support
//
STDMETHODIMP CBinDump::GetCurFile(LPOLESTR * ppszFileName,AM_MEDIA_TYPE *pmt)
{
    CheckPointer(ppszFileName, E_POINTER);
    *ppszFileName = NULL;
    if (m_pFileName != NULL) {
        *ppszFileName = (LPOLESTR)
        QzTaskMemAlloc(sizeof(WCHAR) * (1+lstrlenW(m_pFileName)));
        if (*ppszFileName != NULL) {
            lstrcpyW(*ppszFileName, m_pFileName);
        }
    }

    if(pmt) {
        ZeroMemory(pmt, sizeof(*pmt));
        pmt->majortype = MEDIATYPE_NULL;
        pmt->subtype = MEDIASUBTYPE_NULL;
    }
    return S_OK;

} // GetCurFile


// Destructor

CBinDump::~CBinDump()
{
    CloseFile();
    delete m_pPin;
    delete m_pFilter;
    delete m_pPosition;
    delete m_pFileName;
}


//
// CreateInstance
//
// Provide the way for COM to create a dump filter
//
CUnknown *CBinDump::CreateInstance(LPUNKNOWN punk, HRESULT *phr)
{
    CBinDump *pNewObject = new CBinDump(punk, phr);
    if (pNewObject == NULL) {
        *phr = E_OUTOFMEMORY;
    }
    return pNewObject;

} // CreateInstance


//
// NonDelegatingQueryInterface
//
// Override this to say what interfaces we support where
//
STDMETHODIMP CBinDump::NonDelegatingQueryInterface(REFIID riid, void ** ppv)
{
    CheckPointer(ppv,E_POINTER);
    CAutoLock lck(&m_Lock);

    // Do we have this interface

    if (riid == IID_IFileSinkFilter) {
        return GetInterface((IFileSinkFilter *) this, ppv);
    } else if (riid == IID_IBaseFilter || riid == IID_IMediaFilter || riid == IID_IPersist) {
	return m_pFilter->NonDelegatingQueryInterface(riid, ppv);
    } else if (riid == IID_IMediaPosition || riid == IID_IMediaSeeking) {
        if (m_pPosition == NULL) {

            HRESULT hr = S_OK;
            m_pPosition = new CPosPassThru(NAME("BinDump Pass Through"),
                                           (IUnknown *) GetOwner(),
                                           (HRESULT *) &hr, m_pPin);
            if (m_pPosition == NULL) {
                return E_OUTOFMEMORY;
            }

            if (FAILED(hr)) {
                delete m_pPosition;
                m_pPosition = NULL;
                return hr;
            }
        }
        return m_pPosition->NonDelegatingQueryInterface(riid, ppv);
    } else {
	return CUnknown::NonDelegatingQueryInterface(riid, ppv);
    }

} // NonDelegatingQueryInterface


//
// OpenFile
//
// Opens the file ready for dumping
//
HRESULT CBinDump::OpenFile()
{
    TCHAR *pFileName = NULL;

    // Is the file already opened
    if (m_hFile != INVALID_HANDLE_VALUE) {
        return NOERROR;
    }

    // Has a filename been set yet
    if (m_pFileName == NULL) {
        return ERROR_INVALID_NAME;
    }

    // Convert the UNICODE filename if necessary

#if defined(WIN32) && !defined(UNICODE)
    char convert[MAX_PATH];
    if(!WideCharToMultiByte(CP_ACP,0,m_pFileName,-1,convert,MAX_PATH,0,0))
        return ERROR_INVALID_NAME;
    pFileName = convert;
#else
    pFileName = m_pFileName;
#endif

    // Try to open the file

    m_hFile = CreateFile((LPCTSTR) pFileName,   // The filename
                         GENERIC_WRITE,         // File access
                         (DWORD) 0,             // Share access
                         NULL,                  // Security
                         CREATE_ALWAYS,         // Open flags
                         (DWORD) 0,             // More flags
                         NULL);                 // Template

    if (m_hFile == INVALID_HANDLE_VALUE) {
        DWORD dwErr = GetLastError();
        return HRESULT_FROM_WIN32(dwErr);
    }
    return S_OK;

} // Open


//
// CloseFile
//
// Closes any dump file we have opened
//
HRESULT CBinDump::CloseFile()
{
    if (m_hFile == INVALID_HANDLE_VALUE) {
        return NOERROR;
    }

    CloseHandle(m_hFile);
    m_hFile = INVALID_HANDLE_VALUE;
    return NOERROR;

} // Open


//
// WriteBinary
//
// Writes the given data into the binary file
//
void CBinDump::WriteBinary(PBYTE pbyte, LONG length)
{
    DWORD dwWritten = 0;

    WriteFile((HANDLE) m_hFile,
              (PVOID) pbyte,
              (DWORD) length,
              &dwWritten, NULL);
} // WriteBinary

void CBinDump::WriteMediaType(const CMediaType *pmt)
{
	HANDLE	hFile;
	WCHAR	filename[MAX_PATH];

	if(pmt==NULL)
		return;

    lstrcpyW(filename, m_pFileName);
	lstrcatW(filename, L".mt");

    hFile=CreateFileW(filename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
	if(hFile!=INVALID_HANDLE_VALUE) 
	{
		char	id[4]="MT0";
		DWORD	dwW;
		ULONG	len;
		BOOL	b;

		WriteFile(hFile, (PVOID)id, 4, &dwW, NULL);
		WriteFile(hFile, (PVOID)pmt->Type(), sizeof(GUID), &dwW, NULL);
		WriteFile(hFile, (PVOID)pmt->Subtype(), sizeof(GUID), &dwW, NULL);
		len=pmt->FormatLength();
		WriteFile(hFile, (PVOID)&len, sizeof(ULONG), &dwW, NULL);
		if(len!=0)
			WriteFile(hFile, (PVOID)pmt->Format(), len, &dwW, NULL);
		WriteFile(hFile, (PVOID)pmt->FormatType(), sizeof(GUID), &dwW, NULL);
		
		len=pmt->GetSampleSize();
		WriteFile(hFile, (PVOID)&len, sizeof(ULONG), &dwW, NULL);

		b=pmt->IsTemporalCompressed();
		WriteFile(hFile, (PVOID)&b, sizeof(BOOL), &dwW, NULL);
		
		CloseHandle(hFile);
    }
}
